;Put this file in your GIMP scripts folder. See this video for instructions: https://youtu.be/3EhP8my8FXs (The default for windows is %appdata%\..\Roaming\GIMP\2.10\scripts\ )
(script-fu-register
	"script-fu-spritesheet-to-animated-gif-preview"	;main function to execute
	"Spritesheet to Animated Gif Preview"			;script's title in File > Create menu
	"Open a spritesheet image, then run this\
	script to quickly create an animated gif\
	out of the spritesheet. This is meant for\
	simple preview animations, not game-ready\
	assets. That's what the spritesheet is for. :)"	;description
	"Medicine Storm"								;author
	"copyright 2020, http://www.medicinestorm.com\
	Medicine Storm, licensed CC-BY 4.0\ 
	https://creativecommons.org/licenses/by/4.0/"	;copyright notice
	"December 15, 2020"								;date created
	""												;image-type this script uses
	SF-IMAGE    	"Image"				0
    SF-DRAWABLE 	"Current Layer"		0
	SF-ADJUSTMENT	"Frame Width" 		'(192 8 1024 1 8 0 SF-SPINNER)	;spin-button: (default value, minimum, maximum, small increment, large increment, decimals, adjustment type)
	SF-ADJUSTMENT	"Frame Height" 		'(192 8 1024 1 8 0 SF-SPINNER)	;spin-button: (default value, minimum, maximum, small increment, large increment, decimals, adjustment type)
	SF-ADJUSTMENT	"Frame Count" 		'(32  2 16384 1 8 0 SF-SPINNER)	;spin-button: (default value, minimum, maximum, small increment, large increment, decimals, adjustment type)
	SF-COLOR		"Background color"	'(72 103 144) ;color parameter (default = "OGA blue" R:72, G:103, B:144)
	SF-TOGGLE		"Auto-save to \"[SPRITESHEET'S PATH]-Preview.gif\"\
(uncheck this option to pick the save path &\
filename yourself. Just be sure to click the 
\"as animation\" checkbox when prompted.)" TRUE							;automatically save the gif animation as the same name as the spritesheet, but with "-Preview.gif" at the end.
)
(script-fu-menu-register
	"script-fu-spritesheet-to-animated-gif-preview"
	"<Image>/File/Create"
)
 
(define ;main function.
	(script-fu-spritesheet-to-animated-gif-preview ;main function begin
		p_imagePrimary
		p_layerSpritesheet
		p_spinFrameWidth
		p_spinFrameHeight
		p_spinFrameCount
		p_colorBackground
		p_toggleAutoSave
	)
	(let*
		(;initialization begin
			(strFilename "")

			(intLoopX 0)
			(intLoopY 0)
			(intSpritesheetWidth 0)
			(intSpritesheetHeight 0)
			(intFramesPerRow 0)
			(intFramesPerColumn 0)
			(intFrameNumber 0)
			(intOffsetX 0)
			(intOffsetY 0)
			(fltProgressValue 0.0)

			(fltProgressIncrement 
				(/ 0.5 p_spinFrameCount)
			)
			
			(imageAnimation
				(car 
					(gimp-image-new
						p_spinFrameWidth
						p_spinFrameHeight
						RGB
					)
				)
			)
			
			(imageOptimized
				(car
					(gimp-image-new
						p_spinFrameWidth
						p_spinFrameHeight
						INDEXED
					)
				)
			)
			
			(layerNewFrame
				(car
					(gimp-layer-new
						imageAnimation
						p_spinFrameWidth
						p_spinFrameHeight
						RGB-IMAGE ;It would be RGBA-IMAGE, but these previews have no transparency by design.
						"Frame"	;placeholder name for this placeholder layer.
						100
						LAYER-MODE-NORMAL-LEGACY
					)
				)
			)
			
			(displayAnimation
				(car
					(gimp-display-new imageAnimation)
				)
			)
		);initialization end
	;main code begin
		(gimp-image-undo-group-start p_imagePrimary)
		(gimp-progress-update fltProgressValue)
		(gimp-progress-set-text "Creating GIF from spritesheet")
	
		;prepare the user-selected background color to be used in the following operations
		(gimp-context-set-background p_colorBackground)

		;get spritesheet dimensions
		(set! intSpritesheetWidth
			(car (gimp-drawable-width p_layerSpritesheet))
		)
		(set! intSpritesheetHeight
			(car (gimp-drawable-height p_layerSpritesheet))
		)
		
		;calculate # of columns based on image width & frame width
		(set! intFramesPerRow
			(inexact->exact ;convert from float to int32, basically. "10.0" to "10"
				(floor ;round it down so it ignores any extra padding at the edge of the spritesheet instead of overshooting the bounds of the image.
					(/ intSpritesheetWidth p_spinFrameWidth) ;devide the width of the image by the width of each frame to get the number of frames in each row.
				)
			)
		)
		
		;calculate # of rows based on image height & frame height
		(set! intFramesPerColumn
			(inexact->exact ;convert from float to int32, basically. "10.0" to "10"
				(floor ;round it down so it ignores any extra padding at the edge of the spritesheet instead of overshooting the bounds of the image.
					(/ intSpritesheetHeight p_spinFrameHeight) ;devide the height of the image by the height of each frame to get the number of frames in each column.
				)
			)
		)
		
		;we don't need the placeholder layer that was initially created in the declaration 
		;section, we just needed the layer-variable to house new frames as they're created.
		(gimp-item-delete layerNewFrame) 
		
		;increment progress bar value
		(set! fltProgressValue 
			(+ fltProgressValue fltProgressIncrement)
		) 
		(gimp-progress-update fltProgressValue)
		
		;iterate through each frame on the spritesheet, left to right, top to bottom. If the number of frames is less than the spritesheet can hold, stop early; they're probably blank frames.
		(while (< intLoopY intFramesPerColumn)
			(set! intLoopX 0)
			(while (and (< intLoopX intFramesPerRow) (< intFrameNumber p_spinFrameCount))    
				;(set! strMessage (string-append strMessage "#" (number->string intFrameNumber) ": (" (number->string intLoopX) ", " (number->string intLoopY) ")" (newline)))
				
				;add new layer with sprite's frame dimensions					
				(set! layerNewFrame
					(car
						(gimp-layer-new
							imageAnimation
							p_spinFrameWidth
							p_spinFrameHeight
							RGB-IMAGE
							(string-append "Frame-" (number->string intFrameNumber))
							100
							LAYER-MODE-NORMAL-LEGACY
						)
					)
				)
				
				;fill the empty frame with the chosen background color
				(gimp-drawable-fill
					layerNewFrame
					FILL-BACKGROUND
				)
				
				;add the new empty frame to the layer stack
				(gimp-image-insert-layer
					imageAnimation
					layerNewFrame
					0
					-1 ;change to 0 to reverse the order of the frames
				)
				
				;calculate location of frame in spritesheet
				(set! intOffsetX 
					(* intLoopX p_spinFrameWidth)
				)
				
				(set! intOffsetY
					(* intLoopY p_spinFrameHeight)
				)
				
				;select the frame on the spritesheet
				(gimp-image-select-rectangle
					p_imagePrimary
					CHANNEL-OP-REPLACE
					intOffsetX
					intOffsetY
					p_spinFrameWidth
					p_spinFrameHeight
				)
				
				;copy the frame from the spritesheet and paste it into the new frame layer
				(gimp-edit-copy p_layerSpritesheet)
				
				(gimp-floating-sel-anchor ;paste the copied selection directly into the new frame
					(car 
						(gimp-edit-paste 
							layerNewFrame
							TRUE
						)
					)
				)
				
				;increment loop variables, frame number, and progress bar.
				(set! intLoopX (+ intLoopX 1))
				(set! intFrameNumber (+ intFrameNumber 1))
				(set! fltProgressValue 
					(+ fltProgressValue fltProgressIncrement )
				) 
				(gimp-progress-update fltProgressValue)
			)
			(set! intLoopY (+ intLoopY 1))
		)
		
		(gimp-progress-set-text "Color indexing takes a minute...")
		(gimp-displays-flush)
		
		;convert the image to indexed color (required by gif)
		(gimp-image-convert-indexed
			imageAnimation
			CONVERT-DITHER-FS-LOWBLEED
			CONVERT-PALETTE-GENERATE
			256
			FALSE
			TRUE
			""
		)
		
		;increment progress
		(gimp-progress-set-text "Creating GIF from spritesheet")
		(gimp-progress-update 0.7)
		
		;optimize animation for gif
		(set! imageOptimized
			(car 
				(plug-in-animationoptimize
					RUN-NONINTERACTIVE
					imageAnimation
					layerNewFrame
				)
			)
		)
		
		;increment progress
		(gimp-progress-update 0.9)
		
		;switch view from raw animation image to optimized gif image
		(gimp-displays-reconnect
			imageAnimation
			imageOptimized
		)
		
		;show all changes 
		(gimp-image-undo-group-end p_imagePrimary)
		(gimp-displays-flush) 

		;identify the first frame for reference when saving as gif
		(set! layerNewFrame
			(car 
				(gimp-image-get-layer-by-name
					imageOptimized
					"Frame-0(100ms)"
				)
			)
		)
		
		;increment progress
		(gimp-progress-set-text "Creating GIF from spritesheet")
		(gimp-progress-update 0.99)
		
		 ;for auto-saving, determine the proper file path based on the spritesheet's path.
		 (if (= p_toggleAutoSave TRUE)
			(begin
				;get the spritesheet's file name, but without the extension.
				(set! strFilename
					(car
						(strbreakup 
							(car 
								(gimp-image-get-filename p_imagePrimary)
							)
							"."
						)
					)
				)
				;make sure it HAS a filename.
				(if (equal? strFilename "")
					(set! p_toggleAutoSave FALSE) ;if the spritesheet has no file path, force RUN-INTERACTIVE
					(set! strFilename ;append suffix and gif extension to auto-save filename.
						(string-append
							strFilename
							"-Preview.gif"
						)
					)	
				)
			)
			() ;if auto-save is not selected, no action is needed here.
		)
		
		(if (= p_toggleAutoSave TRUE)
			(begin
				(file-gif-save ;save the file to the auto-save path. No need to bother the user with gif settings.
					RUN-NONINTERACTIVE
					imageOptimized
					layerNewFrame
					strFilename
					strFilename
					FALSE
					TRUE
					100
					1
				)
				;tell the user where the file was saved.
				(gimp-message 
					(string-append 
						"SUCCESS! Animation saved to: \
"
						strFilename
					)
				)
			)
			(begin 
				(file-gif-save ;the user needs to select the file's name and where it should be saved.
					RUN-INTERACTIVE
					imageOptimized
					layerNewFrame
					strFilename
					strFilename
					FALSE
					TRUE
					100
					1
				)
			)
		)
		
		(gimp-display-delete displayAnimation)
	);main code end
);main function end